# -*- coding: utf-8 -*-
from django.utils.translation import gettext_lazy as _
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.response import Response
from thx.api.mixins import ThuxModelMixin

from thx.role.core.views import ThuxRoleViewSetMixin

from ... import models
from . import serializers, filters, permissions

STATUS_CODES = ((0, _('disable')), (1, _('enable')),)


# COUNTRY
class CountryViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    CountryViewSet
    -----------

    :action list: Return the country list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given country.
        `Profiles`: `superuser`, `staff`

    :action light: Return the country list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new country instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a country instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a country instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a country instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.Country.objects.all()
    serializer_class = serializers.CountrySerializer
    filter_backends = (filters.CountryFilterBackend,)
    permission_classes = (permissions.CountryPermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a country instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a country instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# ADMINISTRATIVE AREA
class AdministrativeAreaViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    AdministrativeAreaViewSet
    -----------

    :action list: Return the administrative area list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given administrative area.
        `Profiles`: `superuser`, `staff`

    :action light: Return the administrative area list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new administrative area instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a administrative area instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a administrative area instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a administrative area instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.AdministrativeArea.objects.all()
    serializer_class = serializers.AdministrativeAreaSerializer
    filter_backends = (filters.AdministrativeAreaFilterBackend,)
    permission_classes = (permissions.AdministrativeAreaPermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a administrative area instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a administrative area instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)
