# -*- coding: utf-8 -*-

from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _
from thx.core.models import CleanModel, DateModel, OrderedModel, StatusModel, UserModel


class Country(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    """
    International Organization for Standardization (ISO) 3166-1 Country list
    """
    iso2_code = models.CharField(
        primary_key=True,
        max_length=2,
        verbose_name=_('ISO alpha-2'),
    )
    iso3_code = models.CharField(
        unique=True,
        max_length=3,
        verbose_name=_('ISO alpha-3'),
    )
    name = models.CharField(
        max_length=128,
        verbose_name=_('official name (CAPS)'),
    )
    printable_name = models.CharField(
        max_length=128,
        verbose_name=_('country name'),
    )
    numcode = models.SmallIntegerField(
        blank=True, null=True,
        verbose_name=_('ISO numeric'),
    )
    continent = models.CharField(
        choices=settings.L10N_CONTINENTS,
        max_length=2,
        verbose_name=_('continent'),
    )
    admin_area = models.CharField(
        choices=settings.L10N_AREAS,
        max_length=2,
        blank=True,
        verbose_name=_('administrative area'),
    )

    class Meta:
        verbose_name = _('country')
        verbose_name_plural = _('countries')
        ordering = ('ordering', 'name',)
        permissions = (
            ("list_country", "Can list country"),
            ("detail_country", "Can detail country"),
            ("enable_country", "Can enable country"),
            ("disable_country", "Can disable country"),
        )

    def __str__(self):
        return self.printable_name


class AdministrativeArea(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    """
    Administrative Area level 1 for a country.
    For the US, this would be the states
    """
    country = models.ForeignKey(
        Country,
        related_name='areas',
        on_delete=models.PROTECT,
        verbose_name=_("country"),
    )
    name = models.CharField(
        max_length=60,
        verbose_name=_('admin area name'),
    )
    abbrev = models.CharField(
        max_length=3,
        blank=True,
        verbose_name=_('postal abbreviation'),
    )

    class Meta:
        verbose_name = _('administrative area')
        verbose_name_plural = _('administrative areas')
        ordering = ('ordering', 'name')
        permissions = (
            ("list_administrativearea", "Can list administrative area"),
            ("detail_administrativearea", "Can detail administrative area"),
            ("enable_administrativearea", "Can enable administrative area"),
            ("disable_administrativearea", "Can disable administrative area"),
        )
        unique_together = ('country', 'name',)

    def __str__(self):
        return self.name

    @property
    def country_name(self):
        return self.country.name
